
# coding: utf-8

#Derive matched cohorts
#Inputs: SleepDataClean.csv, GRID_AllPhecodeByVisit, GRIDDemographics.csv, GRID_LastVisit
#Outputs: MatchedNonSleepGRIDs.csv, PhecodeProportions.csv, Summaries.txt
import pandas as pd
import numpy as np
import random, re, sys
from scipy import stats
from itertools import chain


Chrono = pd.read_csv('SleepDataClean.csv')

GRIDs = list(set(Chrono.GRID))

#Read in list of all GRIDs' phecodes
GRIDCodes = pd.read_csv('GRID_AllPhecodeByVisit')

Demos = pd.read_csv('GRIDDemographics.csv')
LastVisits = pd.read_csv('GRID_LastVisit')

#For matching to the sleep cohort, drop NA in demographics and convert non-white and non-blacks to O
Demos = Demos[~Demos.SD_RACE.isna()]
Demos.SD_RACE[(Demos.SD_RACE=='I') | (Demos.SD_RACE=='N') | (Demos.SD_RACE=='U') | (Demos.SD_RACE=='A')] = 'O'

#Add date (then age) of last visit
Demos = Demos.merge(LastVisits, on = 'GRID')

Phecodes = list(set(GRIDCodes.PHECODE))

#Match by age? Start with calculating the visit age for each entry in GRIDCodes
GRIDCodes = GRIDCodes.merge(Demos, on = 'GRID')

#For every phecode in the history, get the age at entry
daylen = pd.to_datetime(GRIDCodes['ENTRY_DATE'], yearfirst = True) - pd.to_datetime(GRIDCodes['DOB'], yearfirst = True)
GRIDCodes['Age'] = [x.days//365 for x in daylen]

daylen = pd.to_datetime(GRIDCodes['LAST_VISIT'], yearfirst = True) - pd.to_datetime(GRIDCodes['DOB'], yearfirst = True)
GRIDCodes['AgeLastVisit'] = [x.days//365 for x in daylen]

#Objective: calculate relative proportions of phecodes for those 4000 GRIDs from sleep study to the rest of the SD
ChronoGridCodes = GRIDCodes[GRIDCodes.GRID.isin(GRIDs)]
NoChronoGridCodes = GRIDCodes[~GRIDCodes.GRID.isin(GRIDs)]

ChronoGridCodesGroup = [x for x in list(ChronoGridCodes.groupby(['GRID']))]
NoChronoGridCodesGroup = [x for x in list(NoChronoGridCodes.groupby(['GRID']))]

#Check that age is included and ordered
ChronoGridCodesGroup = [x[1].sort_values('Age') for x in ChronoGridCodesGroup]
NoChronoGridCodesGroup = [x[1].sort_values('Age') for x in NoChronoGridCodesGroup]

#build a DF of nonchronos by GRID, minyear and visitnum
NoChronoMatch = pd.DataFrame([[x.GRID.values[0] for x in NoChronoGridCodesGroup], [max(x.Age) for x in NoChronoGridCodesGroup],
                              [x.GENDER_EPIC.values[0] for x in NoChronoGridCodesGroup], 
                              [x.SD_RACE.values[0] for x in NoChronoGridCodesGroup],
                              [x.AgeLastVisit.values[0] for x in NoChronoGridCodesGroup],
                              [len(x) for x in NoChronoGridCodesGroup], 
                              [int(max(x.ENTRY_DATE)[0:4]) - int(min(x.ENTRY_DATE)[0:4]) for x in NoChronoGridCodesGroup]])

NoChronoMatch = NoChronoMatch.T
NoChronoMatch.columns = ['GRID', 'MaxAge', 'Gender', 'Race', 'AgeLastVisit', 'VisitNum', 'RecordLength']

#Matching approach
GridList = []
for grid in range(len(ChronoGridCodesGroup)):
    
#....   get their number of visits and year interval
    NumVisits = len(ChronoGridCodesGroup[grid])
    MinYear = min(ChronoGridCodesGroup[grid].ENTRY_DATE)
    MaxYear = max(ChronoGridCodesGroup[grid].ENTRY_DATE)
    MinYear = MinYear[0:4]
    MaxYear = MaxYear[0:4]
    RecordLength = int(MaxYear) - int(MinYear)
    Gender = ChronoGridCodesGroup[grid].GENDER_EPIC.values[0]
    Race = ChronoGridCodesGroup[grid].SD_RACE.values[0]
    LastAge = ChronoGridCodesGroup[grid].AgeLastVisit.values[0]

    MatchList = NoChronoMatch.GRID[(NoChronoMatch.RecordLength.values == RecordLength)
                                   & (NoChronoMatch.AgeLastVisit == LastAge) & (NoChronoMatch.Gender == Gender)
                                   & (NoChronoMatch.Race == Race)]
    #Now you want to add a random GRID not contained in the growing list of matching GRIDs
    try:
        GridList.append(random.choice([z for z in MatchList if z not in GridList]))
    except:
        GridList.append('')
        continue

#Write list of GRIDs of a matched cohort to csv
GridListExpo = pd.DataFrame([GridList]).T
GridListExpo.columns = ['GRID']
GridListExpo.to_csv('CohortMatching_Descriptives/MatchedNonSleepGRIDs.csv')

NoChronoGridCodesGroupMatch = [x for x in NoChronoGridCodesGroup if x.GRID.values[0] in GridList]

ChronoGridCodesGroupVisit = []
for grid in ChronoGridCodesGroup:
    ChronoGridCodesGroupVisit.append([x for x in grid.groupby(['ENTRY_DATE'])])

NoChronoGridCodesGroupVisit = []
for grid in NoChronoGridCodesGroupMatch:
    NoChronoGridCodesGroupVisit.append([x for x in grid.groupby(['ENTRY_DATE'])])

#Update grid list
NoChronoGridCodes = NoChronoGridCodes[NoChronoGridCodes.GRID.isin(GridList)]

ChronoNames = [['GRID'], Phecodes]
ChronoNames = list(chain.from_iterable(ChronoNames))
NoChronoNames = [['GRID'], Phecodes]
NoChronoNames = list(chain.from_iterable(NoChronoNames))

ChronoGRIDs = list(set(ChronoGridCodes.GRID))
NoChronoGRIDs = list(set(NoChronoGridCodes.GRID))

ChronoOutput = dict()
for GridSet in ChronoGridCodesGroup:
    Grid = GridSet.GRID.values[0]
    Codes = GridSet[GridSet.PHECODE.duplicated(keep=False)].PHECODE.values
    #Ensuring that the phecode comes up at least twice!
    Codes = list(set([x for x in Codes if len(set(GridSet.ENTRY_DATE[GridSet.PHECODE==x]))>1]))
    ChronoOutput[Grid] = Codes

NoChronoOutput = dict()
for GridSet in NoChronoGridCodesGroupMatch:
    Grid = GridSet.GRID.values[0]
    Codes = GridSet[GridSet.PHECODE.duplicated(keep=False)].PHECODE.values
#    #Ensuring that the phecode comes up at least twice!
    Codes = list(set([x for x in Codes if len(set(GridSet.ENTRY_DATE[GridSet.PHECODE==x]))>1]))
    NoChronoOutput[Grid] = Codes

#Get a proportion for every phecode in ChronoOutput
ChronoProportion = dict()
for code in Phecodes:
    counter = 0
    for grid in ChronoOutput:
        if code in ChronoOutput[grid]:
            counter += 1
    ChronoProportion[code] = ((counter/len(ChronoOutput) * 100))

#Get a proportion for every phecode in NoChronoOutput
NoChronoProportion = dict()
for code in Phecodes:
    counter = 0
    for grid in NoChronoOutput:
        if code in NoChronoOutput[grid]:
            counter += 1
    NoChronoProportion[code] = ((counter/len(NoChronoOutput) * 100))

Output = pd.DataFrame([ChronoProportion.keys(), ChronoProportion.values()]).T
Output.columns = ['PheCode', 'ChronoProportion']
temp = pd.DataFrame([NoChronoProportion.keys(), NoChronoProportion.values()]).T
temp.columns = ['PheCode', 'NoChronoProportion']
Output = Output.merge(temp ,on = 'PheCode')

Output['Difference'] = Output['ChronoProportion'] - Output['NoChronoProportion']

Output.to_csv('CohortMatching_Descriptives/PhecodeProportions.csv')

#1 - what are the total phecodes per person in each population?
#2 - what are the total number of visits per person in each population?
#Report summary statistics: everything called after print is written to file
sys.stdout = open('CohortMatching_Descriptives/Summaries.txt', 'a')
print("Number of matched grids:", len(set(GridList)))

#1a - total unique phecodes per person - sort of like the phecode proportion test done last week
TotalChronoPheCodesSet = [len(set(x.PHECODE)) for x in ChronoGridCodesGroup]
TotalNoChronoPheCodesSet = [len(set(x.PHECODE)) for x in NoChronoGridCodesGroupMatch]
print("Unique codes per person t-test:", stats.ttest_ind(TotalChronoPheCodesSet, TotalNoChronoPheCodesSet, equal_var = True))

print("Sleep Clinic Visitors Total Unique Phecodes", np.mean(TotalChronoPheCodesSet), np.std(TotalChronoPheCodesSet))
print("All Other GRIDS' Total Unique Phecodes", np.mean(TotalNoChronoPheCodesSet), np.std(TotalNoChronoPheCodesSet))

#2 - what are the total number of visits per person in each population?
TotalChronoVisits = [len(x) for x in ChronoGridCodesGroupVisit]
TotalNoChronoVisits = [len(x) for x in NoChronoGridCodesGroupVisit]
print("Total number of visits t-test:", stats.ttest_ind(TotalChronoVisits, TotalNoChronoVisits, equal_var = True)) #Welch's test

print("Sleep Clinic Visitors Total Visits", np.mean(TotalChronoVisits), np.std(TotalChronoVisits))
print("All Other GRIDS' Total Visits", np.mean(TotalNoChronoVisits), np.std(TotalNoChronoVisits))

