#Piecing together findings from the sleep regression modeling
#Input: SleepDataClean.csv
#Outputs: Fig2.pdf, Fig3.pdf

#Using the rms package for ordinal regression modeling
library(mgcv)
library(readr)
library(rms)
library(ggplot2)
library(cowplot)
library(dplyr)
library(circular)

gendercolors = c('#fc8d62', '#8da0cb')
racecolors = c('#66c2a5', '#fc8d62', '#8da0cb')

#filepath = file.path('..')
filepath = getwd()
dir.create(path = file.path(filepath, 'SleepPhenotypeModels'))


Chrono = read_csv(file = file.path(filepath, 'SleepDataClean.csv'))

#Store for later
BMILow = summary(Chrono$BMI)[[2]]
BMIMed = summary(Chrono$BMI)[[4]]
BMIHigh = summary(Chrono$BMI)[[5]]

#Log-transform BMI
Chrono$BMI = log10(Chrono$BMI)

Chrono = rename(Chrono, Gender = GENDER_EPIC)
Chrono = rename(Chrono, Age = VisitAge)
Chrono = rename(Chrono, Race = SD_RACE)

#Chosen models from sleep durations
WeekdayDurationModel = orm(Weekday.Sleep.Duration ~ Gender + Race + rcs(Age, 4), 
                            data = Chrono, x = TRUE, y = TRUE)
WeekendDurationModel = orm(Weekend.Sleep.Duration ~ Gender + Race + rcs(Age, 3), 
                            data = Chrono, x = TRUE, y = TRUE)

#Prep plots - weekday
mWeekdaySleep = Mean(WeekdayDurationModel) #Back to sleep hour scale
weekdaylogitpreds = Predict(WeekdayDurationModel, Age = c(seq(18, 88, 1)), Gender = c('M', 'F'), Race = c('W', 'B', 'O'),
                            fun = mWeekdaySleep)

#Additional plotting by races, gender at different values of BMI
weekdaylogitgender = aggregate(. ~ Age + Gender, data = weekdaylogitpreds, FUN = "mean")
weekdaylogitgender$Gender = as.character(unlist(weekdaylogitgender$Gender))
weekdaylogitgender$Gender[weekdaylogitgender$Gender == "M"] = "Male"
weekdaylogitgender$Gender[weekdaylogitgender$Gender == "F"] = "Female"

GGWeekdaySleepFacetGender = ggplot(weekdaylogitgender, aes(x = Age, y = yhat, colour = Gender)) + geom_line(size = 1.2) +
  geom_ribbon(aes(ymin = lower, ymax = upper, fill = Gender), colour = NA, alpha = 0.2) + theme_bw() +
  ylab('Sleep duration (h)') + xlab('') + ggtitle("Weekday")+
  ylim(c(7,10.1))+
  scale_fill_manual(values = gendercolors)+
  theme(axis.text = element_text(size = 8), axis.title = element_text(size = 9, face = "bold"), 
        legend.position = "none",
        title = element_text(size = 9.5, face = "bold"), plot.title = element_text(hjust = 0.5),
        plot.margin = margin(t = 0.1, r = 0.1, b = -0.1, l = 0.05, unit = 'in'))

weekdaylogitrace = aggregate(. ~ Age + Race, data = weekdaylogitpreds, FUN = "mean")
weekdaylogitrace$Race = as.character(unlist(weekdaylogitrace$Race))
weekdaylogitrace$Race[weekdaylogitrace$Race == "W"] = "White"
weekdaylogitrace$Race[weekdaylogitrace$Race == "B"] = "Black"
weekdaylogitrace$Race[weekdaylogitrace$Race == "O"] = "Other"

GGWeekdaySleepFacetRace = ggplot(weekdaylogitrace, aes(x = Age, y = yhat, colour = Race)) + geom_line(size = 1.2) +
  geom_ribbon(aes(ymin = lower, ymax = upper, fill = Race), colour = NA, alpha = 0.15) + theme_bw() + ggtitle("") +
  ylab('Sleep duration (h)') + xlab('Age at visit (y)') + 
  ylim(c(7,10.1))+
  scale_fill_manual(values = racecolors)+
  theme(axis.text = element_text(size = 8), axis.title = element_text(size = 9, face = "bold"), 
        legend.position = "none",
        title = element_text(size = 9.5, face = "bold"), plot.title = element_text(hjust = 0.5),
        plot.margin = margin(t = 0.0, r = 0.1, b = 0.0, l = 0.05, unit = 'in'))

#Prep plots - weekend
mWeekendSleep = Mean(WeekendDurationModel) #Back to sleep hour scale
weekendlogitpreds = Predict(WeekendDurationModel, Age = c(seq(18, 88, 1)), Gender = c('M', 'F'), Race = c('W', 'B', 'O'),
                            fun = mWeekendSleep)

#Additional plotting by races, gender at different values of BMI
weekendlogitgender = aggregate(. ~ Age + Gender, data = weekendlogitpreds, FUN = "mean")
weekendlogitgender$Gender = as.character(unlist(weekendlogitgender$Gender))
weekendlogitgender$Gender[weekendlogitgender$Gender == "M"] = "Male"
weekendlogitgender$Gender[weekendlogitgender$Gender == "F"] = "Female"

GGWeekendSleepFacetGender = ggplot(weekendlogitgender, aes(x = Age, y = yhat, colour = Gender)) + geom_line(size = 1.2) +
  geom_ribbon(aes(ymin = lower, ymax = upper, fill = Gender), colour = NA, alpha = 0.2) + theme_bw() + ggtitle("") +
  ylab('') + xlab('') + ggtitle('Weekend')+
  ylim(c(7,10.1))+
  scale_fill_manual(values = gendercolors)+
  theme(axis.text = element_text(size = 8), axis.title = element_text(size = 9, face = "bold"), 
        legend.title = element_text(size = 9, face = 'bold'), legend.text = element_text(size = 8),
        legend.box.margin = margin(-10, -10, -10, -10),
        title = element_text(size = 9.5, face = "bold"), plot.title = element_text(hjust = 0.5),
        plot.margin = margin(t = 0.1, r = 0.1, b = -0.1, l = 0.05, unit = 'in'))

weekendlogitrace = aggregate(. ~ Age + Race, data = weekendlogitpreds, FUN = "mean")
weekendlogitrace$Race = as.character(unlist(weekendlogitrace$Race))
weekendlogitrace$Race[weekendlogitrace$Race == "W"] = "White"
weekendlogitrace$Race[weekendlogitrace$Race == "B"] = "Black"
weekendlogitrace$Race[weekendlogitrace$Race == "O"] = "Other"

GGWeekendSleepFacetRace = ggplot(weekendlogitrace, aes(x = Age, y = yhat, colour = Race)) + geom_line(size = 1.2) +
  geom_ribbon(aes(ymin = lower, ymax = upper, fill = Race), colour = NA, alpha = 0.15) + theme_bw() + ggtitle("") +
  ylab('') + xlab('Age at visit (y)') + 
  ylim(c(7,10.1))+
  scale_fill_manual(values = racecolors)+
  theme(axis.text = element_text(size = 8), axis.title = element_text(size = 9, face = "bold"), 
        legend.title = element_text(size = 9, face = 'bold'), legend.text = element_text(size = 8),
        legend.box.margin = margin(-10, -4.5, -10, -10),
        title = element_text(size = 9.5, face = "bold"), plot.title = element_text(hjust = 0.5),
        plot.margin = margin(t = 0.0, r = 0.1, b = 0.0, l = 0.05, unit = 'in'))

SubPlot1 = plot_grid(GGWeekdaySleepFacetGender, GGWeekendSleepFacetGender,
                     labels = c('A', 'B'), align = 'h', nrow = 1, vjust = 1.75, hjust = -0.2, rel_widths = c(1, 1.2))
SubPlot2 = plot_grid(GGWeekdaySleepFacetRace, GGWeekendSleepFacetRace,
                     labels = c('C', 'D'), align = 'h', nrow = 1, vjust = 1.5, hjust = -0.2, rel_widths = c(1, 1.2))
SleepDurationCow = plot_grid(SubPlot1, SubPlot2, align = 'vh', nrow = 2)

ggsave(filename = "Fig2.pdf", plot = SleepDurationCow, path = file.path(filepath, 'SleepPhenotypeModels'), width = 6, height = 5)


#Social lag and midpoint (weekend yields interesting model)
SocialLagModel = orm(Social.Lag.Midpoint ~ Gender + Race + rcs(Age, 3), 
                      data = Chrono, x = TRUE, y = TRUE)
WeekendMidModel = orm(Weekend.Sleep.MidpointAdj ~ Gender + Race + rcs(Age, 5) + rcs(Age, 5)*BMI + BMI + rcs(Age, 5)*Gender, 
                       data = Chrono, x = TRUE, y = TRUE)

mSocialSleep = Mean(SocialLagModel) #Back to sleep hour scale
Sociallogitpreds = Predict(SocialLagModel, Age = c(seq(18, 88, 1)), Gender = c('M', 'F'), Race = c('W', 'B', 'O'),
                           fun = mSocialSleep)

#Plot population means, if we only want age
Sociallogitpop = aggregate(. ~ Age, data = Sociallogitpreds, FUN = "mean")

GGSocialSleepPop = ggplot(Sociallogitpop, aes(x = Age, y = yhat)) + geom_line(size = 1.2) +
  geom_ribbon(aes(ymin = lower, ymax = upper), colour = NA, alpha = 0.2) + theme_bw() + ggtitle("") +
  ylab('Social sleep lag (h)') + xlab('Age at visit (y)') + 
  theme(axis.text = element_text(size = 9), axis.title = element_text(size = 10, face = "bold"), 
        legend.title = element_text(size = 10, face = 'bold'), legend.text = element_text(size = 9),
        title = element_text(size = 11, face = "bold"), plot.title = element_text(hjust = 0.5),
        plot.margin = margin(t = -0.15, r = 0.1, b = 0.05, l = 0.05, unit = 'in'))

#Midpoint model - race has near-zero effect, but gender*age across BMI levels is worth showing
mWeekendMidSleep = Mean(WeekendMidModel) #Back to sleep hour scale
Weekendlogitpreds = Predict(WeekendMidModel, Age = c(seq(18, 88, 1)), Gender = c('M', 'F'), Race = c('W', 'B', 'O'),
                            BMI = c(log10(BMILow), log10(BMIMed), log10(BMIHigh)), fun = mWeekendMidSleep)

#Additional plotting by races, gender at different values of BMI
Weekendmidlogitgender = aggregate(. ~ Age + Gender + BMI, data = Weekendlogitpreds, FUN = "mean")
Weekendmidlogitgender$BMI = round(10^(Weekendmidlogitgender$BMI), 1)
Weekendmidlogitgender$BMILabel = paste0('BMI=', Weekendmidlogitgender$BMI)
Weekendmidlogitgender$Gender = as.character(unlist(Weekendmidlogitgender$Gender))
Weekendmidlogitgender$Gender[Weekendmidlogitgender$Gender == "M"] = "Male"
Weekendmidlogitgender$Gender[Weekendmidlogitgender$Gender == "F"] = "Female"

#Readjust to clock time
adjustment = mean.circular(as.circular(Chrono$Weekend.Sleep.Midpoint, units = 'hours'), na.rm = TRUE)[[1]]
Weekendmidlogitgender$yhatadj = Weekendmidlogitgender$yhat + adjustment
Weekendmidlogitgender$loweradj = Weekendmidlogitgender$lower + adjustment
Weekendmidlogitgender$upperadj = Weekendmidlogitgender$upper + adjustment

GGWeekendMidFacetGender = ggplot(Weekendmidlogitgender, aes(x = Age, y = yhatadj, colour = Gender)) + geom_line(size = 1.2) +
geom_ribbon(aes(ymin = loweradj, ymax = upperadj, fill = Gender), colour = NA, alpha = 0.2) + theme_bw() + ggtitle("") +
ylab(paste('Weekend midpoint', '\n', '(clock time)')) + xlab('Age at visit (y)') +
scale_y_continuous(limits = c(-0.5, 4.5), breaks = c(0, 1, 2, 3, 4), labels = c('00:00', '01:00', '02:00', '03:00', '04:00'))+
scale_fill_manual(values = gendercolors)+
facet_wrap(~ BMILabel, scales = 'fixed')+#, scales = 'free')+
theme(axis.text = element_text(size = 9), axis.title = element_text(size = 10, face = "bold"),
legend.title = element_text(size = 9, face = 'bold'), legend.text = element_text(size = 8),
legend.box.margin = margin(-10, -3, -10, -10),
title = element_text(size = 11, face = "bold"), plot.title = element_text(hjust = 0.5),
plot.margin = margin(t = -0.15, r = 0.0, b = 0.05, l = 0.05, unit = 'in'))

SleepSocialMidCow = plot_grid(GGSocialSleepPop, GGWeekendMidFacetGender, labels = "AUTO",
axis = 'bt', align = 'h', vjust = c(1.1, 1.1), nrow = 1, rel_widths = c(1, 2.5))

ggsave(filename = "Fig3.pdf", plot = SleepSocialMidCow, path = file.path(filepath, 'SleepPhenotypeModels'), width = 8, height = 2.75)
