#Run overlapped GWAS across exome and mega
#Run GWAS on sleep phenotypes - Mega - also european
#Inputs: SleepDataClean.csv, PublishedChronoSNPsProxies.csv, FinalGridSnpMatExome.csv, 
#.. FinalGridSnpMatMega.csv, Mega_IDs.csv, IDs_Euro.txt, ExomeIDs.csv, Exome_ancestry.txt
#IDs_Euro.txt and Exome_ancestry.txt generated by others in the Denny group

#Outputs: WeekdaySleepDurationTable.csv, WeekendSleepDurationTable.csv, Fig5.pdf

#Targeted GWAS - using the list of published SNPs in Lane, Jones, Hu et al.
#.. and other SNPs in LD (R2>0.80, https://analysistools.nci.nih.gov/LDlink/)

library(ggplot2)
library(readr)
library(reshape2)
library(dplyr)
library(cowplot)
library(rms) #With a targeted GWAS, we could use the orm models on our sleep phenotypes

#filepath = file.path('..')
filepath = getwd()

#Store results
dir.create(path = file.path(filepath, 'GWAS'))

#Read in sleep phenotypes
Chrono = read_csv(file = file.path(filepath, 'SleepDataClean.csv'))

#Matrix of SNPs on the BeadChip
MegaSnpGrid = read_csv(file.path(filepath, 'GWAS', 'FinalGridSnpMatMega.csv'))
ExomeSnpGrid = read_csv(file.path(filepath, 'GWAS', 'FinalGridSnpMatExome.csv'))

#Use this file for GRID filtering
MegaAncestry = read_delim(file.path(filepath, 'IDs_Euro.txt'), delim = '\t')
ExomeAncestry = read_delim(file.path(filepath, 'Exome_Ancestry.txt'), delim = '\t')

#IDs for rs to exm match
ExomeIDs = read_csv(file.path(filepath, 'ExomeIDs.csv'))
MegaIDs = read_csv(file.path(filepath, 'Mega_IDs.csv'))

#Read in the curated list of published SNPs and their "proxy" SNPs in high LD
PubSNPs = read_csv(file = file.path(filepath, 'PublishedChronoSNPsProxies.csv'))
#Concat all entries from the table
PubSNPs = melt(PubSNPs, id.vars = 'Mega')$value
PubSNPs = unique(PubSNPs[grep('rs', PubSNPs)]) #725 SNPs that were either published or in high LD

#Convert to exm
PubSNPsExm = MegaIDs$SNP[MegaIDs$rsID %in% PubSNPs]

#Trim SnpGrid now to reduce size
ExomeSnpGrid = cbind(ExomeSnpGrid$GRID, ExomeSnpGrid[, c(colnames(ExomeSnpGrid) %in% PubSNPsExm)])
colnames(ExomeSnpGrid)[1] = 'GRID'
MegaSnpGrid = cbind(MegaSnpGrid$GRID, MegaSnpGrid[, c(colnames(MegaSnpGrid) %in% PubSNPsExm)])
colnames(MegaSnpGrid)[1] = 'GRID'

#Merge by GRIDs to bring in the SNP info and Ancestry
ExomeComboDF = merge(Chrono, ExomeAncestry, by.x = 'GRID', by.y = 'GRID')
ExomeComboDF = merge(ExomeComboDF, ExomeSnpGrid, by.x = 'GRID', by.y = 'GRID')
MegaComboDF = merge(Chrono, MegaAncestry, by.x = 'GRID', by.y = 'FID')
MegaComboDF = merge(MegaComboDF, MegaSnpGrid, by.x = 'GRID', by.y = 'GRID')

#List of SNPs - based on exome snps (which are all on the mega)
SnpList = colnames(ExomeComboDF)[grep('exm|rs|C-T', colnames(ExomeComboDF))]
SnpList = as.character(unlist(SnpList))

#Now trim the mega data based on what is in the exome
MegaComboDF = MegaComboDF[, which(colnames(MegaComboDF) %in% colnames(ExomeComboDF))]
#Intersect lists of GRIds
intersectlist = unique(intersect(MegaComboDF$GRID, ExomeComboDF$GRID))

#Define values used for associations - exome vs mega calls
for(grid in intersectlist){
  for(snp in SnpList){
    #If values are NA - set it to the Exome value since its viewed as more of the gold standard right now
    if(is.na(MegaComboDF[MegaComboDF$GRID==grid, colnames(MegaComboDF)==snp])){
      MegaComboDF[MegaComboDF$GRID==grid, colnames(MegaComboDF)==snp] = MegaComboDF[ExomeComboDF$GRID==grid, colnames(ExomeComboDF)==snp]
    }
    #If values are real (0, 1, 2), and they don't match across the grid and snp of interest, see the values to Exome
    if(is.finite(unique(ExomeComboDF[ExomeComboDF$GRID==grid, colnames(ExomeComboDF)==snp])) & is.finite(unique(MegaComboDF[MegaComboDF$GRID==grid, colnames(MegaComboDF)==snp]))){
      if(unique(ExomeComboDF[ExomeComboDF$GRID==grid, colnames(ExomeComboDF)==snp]) != unique(MegaComboDF[MegaComboDF$GRID==grid, colnames(MegaComboDF)==snp])){
        ExomeComboDF[ExomeComboDF$GRID==grid, colnames(ExomeComboDF)==snp] = NA
        MegaComboDF[MegaComboDF$GRID==grid, colnames(MegaComboDF)==snp] = ExomeComboDF[ExomeComboDF$GRID==grid, colnames(ExomeComboDF)==snp]
      }
    }
  }
}

#Merge and drop duplicate rows
ComboDF = rbind(MegaComboDF, ExomeComboDF[, names(MegaComboDF)])
ComboDF = unique(ComboDF) # Now 568 rows

#Remove SNPs which have frequency below 5% within this cohort
for (i in 1 : length(SnpList)){
  if(sum(ComboDF[, which(colnames(ComboDF) %in% SnpList[i])] != 0, na.rm = T) <= (nrow(ComboDF) * 0.01)) {
    ComboDF = ComboDF[, -which(colnames(ComboDF) %in% c(SnpList[i]))]
  }
}
#exm129312 is MAF < 0.05

#Update SNP list based on cutoffs
SnpList = colnames(ComboDF)[grep('exm|rs|C-T', colnames(ComboDF))]
SnpList = as.character(unlist(SnpList))

#One naming concern
SnpList = gsub('exm-', '', SnpList)
colnames(ComboDF) = gsub('exm-', '', colnames(ComboDF))
ExomeIDs$SNP = gsub('exm-', '', ExomeIDs$SNP)

#List of Sleep variables for PheWAS - to feed into function
SleepVariables=c("Weekday.Sleep.Duration", "Weekend.Sleep.Duration")

#Function to cycle through GWAS for the sleep variables
GWASFn = function(sleepdata, sleepvar, snps) {
  
  #PValList = matrix(nrow = length(snps), ncol = 4)
  PValList = matrix(nrow = length(snps), ncol = 3)
  
  #For every phecode..
  for(i in 1 : length(snps)) {
    SNP = snps[i]
    
    f = paste(sleepvar, "~", SNP, "+ VisitAge + GENDER_EPIC")
    err = try(GWASModel <- orm(as.formula(f), data = sleepdata), silent = TRUE)
    if(inherits(err, "try-error")) {
      PValList[i, ] = NaN
    } else {
      
      #Extract model info - orm
      Coef = GWASModel$coefficients[SNP][[1]] #Extract coefficient
      StErr = sqrt(diag(GWASModel$var))[SNP][[1]] #use Var-Cov matrix to extract coefficient S.E.
      Z = 1-pchisq(coef(GWASModel)[SNP]^2/vcov(GWASModel)[2,2], 1) #gives Pr(>|Z|)

      colnames(PValList) = c('Coef', 'S.E.', 'Pr(>|Z|)')
      
      #Orm fill
      if(length(Coef) > 0) {
        PValList[i, 'Coef'] = Coef
        PValList[i, 'S.E.'] = StErr
        PValList[i, 'Pr(>|Z|)'] = Z
      }
      if(length(Coef) == 0) {
        PValList[i, ] = NA
      }
    }
  }
  
  #Processing/export
  SnpTrimExport = cbind(1 : nrow(PValList), PValList)
  SnpTrimExport = as.data.frame(SnpTrimExport)
  SnpTrimExport = cbind(SnpList,SnpTrimExport)
  SnpTrimExport = SnpTrimExport[, c(1, 3, 4, 5)]
  SnpTrimExport = SnpTrimExport[order(SnpTrimExport$`Pr(>|Z|)`), ]
  
  #Multiple testing correction - silly for 5 snps
  SnpTrimExport$FDR = p.adjust(SnpTrimExport$`Pr(>|Z|)`, method = 'fdr')
  SnpTrimExport$Bonf = p.adjust(SnpTrimExport$`Pr(>|Z|)`, method = 'bonf')
  
  #Add name
  SnpTrimExport$SnpList = as.character(unlist(SnpTrimExport$SnpList))
  SnpTrimExport = merge(SnpTrimExport, ExomeIDs, by.x = 'SnpList', by.y = 'SNP')
  SnpTrimExport = SnpTrimExport[order(SnpTrimExport$`Pr(>|Z|)`), ]
  
  #write_csv(x = SnpTrimExport, path = file.path(filepath, 'results', paste0(gsub('\\.', '', sleepvar), "RawTable.csv")))

  #Alt writeout for a neat table
  AltOut = select(SnpTrimExport, rsID, Coef, S.E., 'Pr(>|Z|)')
  colnames(AltOut) = c('SNP', 'Coef', 'S.E.', 'PValue')
  write_csv(x = AltOut, path = file.path(filepath, 'GWAS', paste0(gsub('\\.', '', sleepvar), "Table.csv")))
  AltOut$Coef = round(AltOut$Coef, 2)
  AltOut$'S.E.' = round(AltOut$'S.E', 2)
  AltOut$'PValue' = round(AltOut$'PValue', 2)

}

#Concern about exm280604 - only one person has value of 2 - spurious statistically speaking
ComboDF$exm280604[ComboDF$exm280604==2] = NA

#Run GWAS on the sleep variables, cycling through and writing results to file
for (var in 1 : length(SleepVariables)) {
  GWASFn(ComboDF, SleepVariables[var], SnpList)
}

#Return the median (plus a jitter) for adding text to boxplot
n_fun <- function(x){
    return(data.frame(y = 2, label = paste0("n=", length(x))))
}

#Plot most significant finding
WeekdayDurPlot = ComboDF[c(!is.na(ComboDF$exm2268759)), ]
WeekdayDurPlot = select(ComboDF, Weekday.Sleep.Duration, exm2268759)
#Allele - TT, TC, CC
WeekdayDurPlot$exm2268759[WeekdayDurPlot$exm2268759=='0'] = 'TT'
WeekdayDurPlot$exm2268759[WeekdayDurPlot$exm2268759=='1'] = 'TC'
WeekdayDurPlot$exm2268759[WeekdayDurPlot$exm2268759=='2'] = 'CC'
WeekdayDurPlot$exm2268759 = as.factor(WeekdayDurPlot$exm2268759)
WeekdayDurPlot$exm2268759 = factor(WeekdayDurPlot$exm2268759, levels = c('TT', 'TC', 'CC'))
WeekdayDurPlot = WeekdayDurPlot[c(!is.na(WeekdayDurPlot$exm2268759)), ]
WeekdayGG = ggplot(WeekdayDurPlot, aes(x = exm2268759, y = Weekday.Sleep.Duration, group = exm2268759))+
geom_boxplot()+
xlab('')+
ylab("Sleep duration (h)")+
ggtitle("Weekday")+
scale_y_continuous(limits = c(2, 17), breaks = seq(4, 16, 4))+
stat_summary(fun.data = n_fun, geom = "text", size = 3)+
theme_bw()+
theme(axis.text = element_text(size = 9), axis.title = element_text(size = 9, face = "bold"),
title = element_text(size = 9, face = "bold"), plot.title = element_text(hjust = 0.5))

WeekendDurPlot = ComboDF[c(!is.na(ComboDF$exm2268759)), ]
WeekendDurPlot = select(ComboDF, Weekend.Sleep.Duration, exm2268759)
WeekendDurPlot$exm2268759[WeekendDurPlot$exm2268759=='0'] = 'TT'
WeekendDurPlot$exm2268759[WeekendDurPlot$exm2268759=='1'] = 'TC'
WeekendDurPlot$exm2268759[WeekendDurPlot$exm2268759=='2'] = 'CC'
WeekendDurPlot$exm2268759 = as.factor(WeekendDurPlot$exm2268759)
WeekendDurPlot$exm2268759 = factor(WeekendDurPlot$exm2268759, levels = c('TT', 'TC', 'CC'))
WeekendDurPlot = WeekendDurPlot[c(!is.na(WeekendDurPlot$exm2268759)), ]
WeekendGG = ggplot(WeekendDurPlot, aes(x = exm2268759, y = Weekend.Sleep.Duration, group = exm2268759))+
geom_boxplot()+
xlab('')+
ggtitle("Weekend")+
stat_summary(fun.data = n_fun, geom = "text", size = 3)+
scale_y_continuous(limits = c(2, 17), breaks = seq(4, 16, 4))+
ylab('')+
theme_bw()+
theme(axis.text = element_text(size = 9), axis.title = element_text(size = 9, face = "bold"),
title = element_text(size = 9, face = "bold"), plot.title = element_text(hjust = 0.5))

SNPCow = plot_grid(WeekdayGG, WeekendGG, labels = 'AUTO')
SNPCow2 = ggdraw(add_sub(SNPCow, paste(ExomeIDs$rsID[ExomeIDs$SNP=='exm2268759']), size = 10,
vpadding=grid::unit(0, "lines"), fontface = 'bold', vjust = -1))

ggsave(filename = "Fig5.pdf", plot = SNPCow2, path = file.path(filepath, 'GWAS'), width = 6, height = 2.8)
