#Correlations of individuals with more than one entry in the sleep data
#Inputs: SleepDataClean.csv
#Outputs: FigS2.pdf
library(reshape2)
library(ggplot2)
library(ggthemes)
library(ggrepel)
library(cowplot)
library(readr)
library(dplyr)
library(circular)
options(warn = -1)

#filepath = file.path('..')
filepath = getwd()

#Read in sleep phenotypes
Chrono = read_csv(file = file.path(filepath, 'SleepDataClean.csv'))

#Grab individuals with more than one entry - everyone has 2 and only 2 entries)
Dups = Chrono$GRID[duplicated(Chrono$GRID)]
ChronoSelect = subset(Chrono, Chrono$GRID %in% Dups)

ChronoSelect = select(ChronoSelect, GRID, Weekday.Wake.Time, Weekday.Bed.Time, Weekend.Wake.Time, Weekend.Bed.Time)

#Adjust bed times for plotting purposes
ChronoSelect$Weekday.Bed.Time[!is.na(ChronoSelect$Weekday.Bed.Time) & ChronoSelect$Weekday.Bed.Time < 12] = ChronoSelect$Weekday.Bed.Time[!is.na(ChronoSelect$Weekday.Bed.Time) & ChronoSelect$Weekday.Bed.Time < 12] + 24
ChronoSelect$Weekend.Bed.Time[!is.na(ChronoSelect$Weekend.Bed.Time) & ChronoSelect$Weekend.Bed.Time < 12] = ChronoSelect$Weekend.Bed.Time[!is.na(ChronoSelect$Weekend.Bed.Time) & ChronoSelect$Weekend.Bed.Time < 12] + 24

X = ChronoSelect[duplicated(ChronoSelect$GRID), ]
#One person has 3 entries, for this purpose of pairwise plotting and correlation analysis, drop that one
X = X[!duplicated(X$GRID), ]

X2 = melt(X)
colnames(X2)[3] = 'X'
Y = ChronoSelect[!duplicated(ChronoSelect$GRID), ]
Y2 = melt(Y)
colnames(Y2)[3] = 'Y'

GG = merge(X2, Y2, by = c('GRID', 'variable'))
GG$variable = gsub("\\.", " ", GG$variable)
GG$variable = gsub("Adj", "", GG$variable)

#Set correlation statistics
for(var in unique(GG$variable)){
  #use circular correlation
  GG$Cor[GG$variable==var] = format(round(cor.circular(x = GG$X[GG$variable==var], y = GG$Y[GG$variable==var]), 2), nsmall = 2)
}


WeekdayBed = GG[GG$variable=='Weekday Bed Time', ]
GGWeekdayBed = ggplot(WeekdayBed, aes(x = X, y = Y)) +
  geom_point()+
  theme_bw()+
  scale_x_continuous(limits = c(19, 36), breaks = c(20, 24, 28, 32, 36), labels = c("20:00", "00:00", "04:00", "08:00", "12:00"))+
  scale_y_continuous(limits = c(19, 28), breaks = c(20, 24, 28), labels = c("20:00", "00:00", "04:00"))+
  xlab("")+
  ylab("Clock time")+
  ggtitle("Weekday Bed Time")+
  #facet_wrap(~ variable, scales = 'free')+#, scales = 'free')+
  geom_text(aes(x = -Inf, y = -Inf, label = Cor), fontface = 'plain', size = 3.5, vjust = -0.5, hjust = -7)+
  theme(axis.text = element_text(size = 9), axis.title = element_text(size = 10, face = "bold"), legend.position = 'none', title = element_text(size = 9, face = "bold"), plot.title = element_text(hjust = 0.5))

WeekendBed = GG[GG$variable=='Weekend Bed Time', ]
GGWeekendBed = ggplot(WeekendBed, aes(x = X, y = Y)) +
  geom_point()+
  theme_bw()+
  scale_x_continuous(limits = c(19, 36), breaks = c(20, 24, 28, 32, 36), labels = c("20:00", "00:00", "04:00", "08:00", "12:00"))+
  scale_y_continuous(limits = c(19, 28), breaks = c(20, 24, 28), labels = c("20:00", "00:00", "04:00"))+
  xlab("")+
  ylab("")+
  ggtitle("Weekend Bed Time")+
  #facet_wrap(~ variable, scales = 'free')+#, scales = 'free')+
  geom_text(aes(x = -Inf, y = -Inf, label = Cor), fontface = 'plain', size = 3.5, vjust = -0.5, hjust = -7)+
  theme(axis.text = element_text(size = 9), axis.title = element_text(size = 10, face = "bold"), legend.position = 'none', title = element_text(size = 9, face = "bold"), plot.title = element_text(hjust = 0.5))

WeekdayWake = GG[GG$variable=='Weekday Wake Time', ]
GGWeekdayWake = ggplot(WeekdayWake, aes(x = X, y = Y)) +
  geom_point()+
  theme_bw()+
  scale_x_continuous(limits = c(0, 20), breaks = c(0, 5, 10, 15, 20), labels = c("00:00", "05:00", "10:00", "15:00", "20:00"))+
  scale_y_continuous(limits = c(0, 15), breaks = c(0, 5, 10, 15), labels = c("00:00", "05:00", "10:00", "15:00"))+
  xlab("Clock time")+
  ylab("Clock time")+
  ggtitle("Weekday Wake Time")+
  #facet_wrap(~ variable, scales = 'free')+#, scales = 'free')+
  geom_text(aes(x = -Inf, y = -Inf, label = Cor), fontface = 'plain', size = 3.5, vjust = -0.5, hjust = -7)+
  theme(axis.text = element_text(size = 9), axis.title = element_text(size = 10, face = "bold"), legend.position = 'none', title = element_text(size = 9, face = "bold"), plot.title = element_text(hjust = 0.5))

WeekendWake = GG[GG$variable=='Weekend Wake Time', ]
GGWeekendWake = ggplot(WeekendWake, aes(x = X, y = Y)) +
  geom_point()+
  theme_bw()+
  scale_x_continuous(limits = c(0, 20), breaks = c(0, 5, 10, 15, 20), labels = c("00:00", "05:00", "10:00", "15:00", "20:00"))+
  scale_y_continuous(limits = c(0, 15), breaks = c(0, 5, 10, 15), labels = c("00:00", "05:00", "10:00", "15:00"))+
  xlab("Clock time")+
  ylab("")+
  ggtitle("Weekend Wake Time")+
  geom_text(aes(x = -Inf, y = -Inf, label = Cor), fontface = 'plain', size = 3.5, vjust = -0.5, hjust = -7)+
  theme(axis.text = element_text(size = 9), axis.title = element_text(size = 10, face = "bold"), legend.position = 'none', title = element_text(size = 9, face = "bold"), plot.title = element_text(hjust = 0.5))


for(person in X$GRID){ #unique grids listed in X$GRID
  daydiff = as.Date(as.character(Chrono$Date[Chrono$GRID==person][1]), format = "%b %d %Y") - as.Date(as.character(Chrono$Date[Chrono$GRID==person][2]), format = "%b %d %Y")
  X$DayDiff[X$GRID==person] = abs(daydiff[[1]])
  print(daydiff)
  print(person)
}

#For some reason a person has a duplicated note on the same date, but with missing entries for sleep behvaiors, which is why it wasn't filtered before
X$DayDiff[X$DayDiff==0] = NA
#Invalid date entry gave erroneous result and can't calculate difference in days.. so for the purpose of plotting just exclude it
X$DayDiff[X$DayDiff>100000] = NA
X$YearDiff = X$DayDiff/365

#Supplementary figure - histogram of visit years
VisitDiffGG = ggplot(X, aes(x = YearDiff)) +
  theme_bw()+
  ylab('Proportion')+
  xlab("Time between visits (years)")+
  stat_ecdf()+
  theme(axis.text = element_text(size = 9), axis.title = element_text(size = 10, face = "bold"), legend.title = element_blank())

CowPlot = plot_grid(GGWeekdayBed, GGWeekendBed, GGWeekdayWake, GGWeekendWake, VisitDiffGG, nrow = 3, labels = "AUTO", align = 'vh', vjust = c(1.25, 1.25, 1.25, 1.25, 1.25))

ggsave(filename = "FigS2.pdf", plot = CowPlot, path = file.path(filepath, 'RecordSummaries'), width = 6, height = 6)
