#Figure plotting for cohort comparisons and phecode prevalence plot
#Input: pheinfo.rda, PhecodeProportions.csv, MatchedNonSleepGRIDs.csv
#Output: FigS3.pdf, PhecodeProportionsSummary.csv

library(ggplot2)
library(ggthemes)
library(ggrepel)
library(readr)
library(dplyr)
library(cowplot)

#filepath = file.path('..')
filepath = getwd()

load(file.path(filepath, "pheinfo.rda"))

#Proportions plot - 1a
Props=read_csv(file = file.path(filepath, "CohortMatching_Descriptives", "PhecodeProportions.csv"))

Props = Props[c(sort(Props$PheCode)), ]

PropsMerge = merge(Props, pheinfo, by.x = "PheCode", by.y = "phecode", all.y = TRUE)
PropsMerge$ChronoProportion[is.na(PropsMerge$ChronoProportion)] = 0
PropsMerge = PropsMerge[!c(duplicated(PropsMerge)), ]

#Get N for proportions test
Chrono = read_csv(file = file.path(filepath, 'SleepDataClean.csv'))
NoChrono = read_csv(file = file.path(filepath, 'CohortMatching_Descriptives', 'MatchedNonSleepGRIDs.csv'))

#Two-proportion z-test
nChr = length(unique(Chrono$GRID)) #number of inidividuals
nNoChr = length(unique(NoChrono$GRID)) #number of inidividuals

#Proportions test
PropsMerge$PValue = c(1:nrow(PropsMerge))
for(i in 1:nrow(PropsMerge)){
  err = try(prop.test(x = c((PropsMerge$ChronoProportion[i]/100*nChr), (PropsMerge$NoChronoProportion[i]/100*nNoChr)), n = c(nChr, nNoChr)), silent = TRUE)
  if(inherits(err, "try-error")) {
    PropsMerge$PValue[i] = NaN
  } else {
    PropsMerge$PValue[i] = prop.test(x = c((PropsMerge$ChronoProportion[i]/100*nChr), (PropsMerge$NoChronoProportion[i]/100*nNoChr)), n = c(nChr, nNoChr))$p.value
  }
}
PropsMerge$FDR = p.adjust(PropsMerge$PValue, method = 'fdr') #Not too important here but add for completeness

Out = select(PropsMerge, PheCode, ChronoProportion, NoChronoProportion, Difference, PValue, FDR, description, group)

colnames(Out) = c('Phecode', 'Sleep Prevalence', 'Matched Prevalence', 'Percent Difference', 'P-value', 'FDR', 'Description', 'Class')

Out = Out[c(order(Out$`Percent Difference`, decreasing = TRUE)),]
write_csv(Out, path = file.path(filepath, 'CohortMatching_Descriptives', 'PhecodeProportionsSummary.csv'))

PropsMerge$Index = rownames(PropsMerge)
PropsMerge$Index = as.numeric(PropsMerge$Index)
TickLocations = c(1 : length(unique(PropsMerge$groupnum)))
for (i in 1 : length(unique(PropsMerge$group))) {
  TickLocations[i] = mean(as.numeric(PropsMerge$Index[grep(unique(PropsMerge$group)[i], PropsMerge$group)]))
}

PhePlotGG = ggplot(PropsMerge, aes(x = Index,y = Difference, colour = color))+
  geom_point(cex = 0.8)+
  theme_bw()+
  ylab(paste('Difference in prevalence between cohorts'))+
  xlab('Phenotype category')+
  geom_text_repel(data = PropsMerge[round(abs(PropsMerge$Difference)) >= 10.0 & PropsMerge$PValue < 0.05, ], aes(label = description,colour = color), size = 3, max.iter = 100000)+
  ylim(c(-1, 60))+
  scale_x_continuous(breaks = TickLocations, labels = unique(PropsMerge$group))+
  theme(axis.text.x = element_text(size = 9, angle = 45, vjust = 1,hjust = 1), 
        plot.margin = margin(t = 0.1, r = 0.1, b = 0.1, l = 0.1, unit = 'in'),
        panel.grid.major = element_line(colour = 'grey98'), panel.grid.minor = element_line(colour = 'grey98'),
        axis.title.x = element_text(size = 12, face = "bold"), axis.title.y = element_text(size = 11, face = "bold"), legend.position = 'none', 
        title = element_text(size = 12, face = "bold"), plot.title = element_text(hjust = 0.5))

ggsave(filename = "FigS3.pdf", plot = PhePlotGG, path = file.path(filepath, 'CohortMatching_Descriptives'), width = 6, height = 5)
