#Run PheWAS on sleep phenotypes
#Inputs: pheinfo.rda, SleepDataClean.csv, GridPhecodes_Min2Cases.csv, phecode_icd9_rolled.csv
#Outputs: FigS4.pdf

library(ggplot2)
library(ggthemes)
library(ggrepel)
library(cowplot)
library(readr)

#############
#Procedure:
#1 - cycle through phecodes' associations with sleep phenotypes as glms()
#2 - Compile test statistics, plot PheWAS plots and qq plots and save data frames to file
##############

#filepath = file.path('..')
filepath = getwd()

load(file.path(filepath, "pheinfo.rda"))
#Map of phecodes
phecodemap = read_csv(file.path(filepath, 'phecode_icd9_rolled.csv'))

#Read in sleep phenotypes
Chrono = read_csv(file = file.path(filepath, 'SleepDataClean.csv'))

#Read phecodes - minimum of 2 cases
GridPhes = read_csv(file.path(filepath, 'GridPhecodes_Min2Cases.csv'))
colnames(GridPhes) = paste0('PheCode', colnames(GridPhes))

ComboDF = merge(Chrono,GridPhes,by.x = 'GRID', by.y = 'PheCodeGRID')

#List of phecodes
PheList = colnames(ComboDF)[grep('[0-9]', colnames(ComboDF))]
PheList = as.character(unlist(PheList))

#Remove phecodes with <1% prevalence
for(phecode in PheList){
  if(sum(ComboDF[, phecode], na.rm = TRUE) < nrow(ComboDF)*0.01){
    ComboDF = ComboDF[, -which(names(ComboDF) %in% phecode)]
  }
}

#Drop merge column names
ComboDF = ComboDF[, -c(grep('X1', colnames(ComboDF)))]

#Update list of phecodes
PheList = colnames(ComboDF)[grep('[0-9]', colnames(ComboDF))]
PheList = as.character(unlist(PheList))


#Generic phewas plotting function
QQPlot = function(DF, phecodesmap, label, groupcall = ''){
  
  label = gsub("\\.", " ", label)
  
  qqdf = data.frame(observed = -log10(DF$`Pr(>|z|)`), expected = -log10(ppoints(length(DF$`Pr(>|z|)`))))
  ggqq = ggplot(qqdf) + 
    geom_point(aes(x = expected, y = observed), size = 0.8) + 
    geom_abline(intercept = 0, slope = 1) +
    theme_bw() + 
    xlab(expression(bold('Expected -log'[10]~'(p)'))) +
    ylab(expression(bold('Observed -log'[10]~'(p)'))) +
    ggtitle(label) +
    theme(axis.text = element_text(size = 9), axis.title = element_text(size = 9, face = "bold"), 
          panel.grid.major = element_line(colour = 'grey98'), panel.grid.minor = element_line(colour = 'grey98'),
          legend.position = 'none', title = element_text(size = 9, face = "bold"), 
          plot.title = element_text(hjust = 0.5))
  return(ggqq)
}

#Generalize the name
ComboDF$Social.Sleep.Lag = ComboDF$Social.Lag.Midpoint

#List of Sleep variables for PheWAS - to feed into function
SleepVariables=c("Weekday.Sleep.Duration", "Weekend.Sleep.Duration", "Social.Sleep.Lag")

#Prep for how to include the exclusion criteria
broken = strsplit(phecodemap$`Excl. Phecodes`, split = '-|,')
for(item in 1:length(broken)){
  broken[[item]] = as.numeric(broken[[item]]) #some phecodes don't have exclusions, NAs will come out
}

#There are upwards of 3 possible exclusion ranges to consider
phecodemap$PhecodeLow1 = c(1:nrow(phecodemap))
phecodemap$PhecodeHigh1 = c(1:nrow(phecodemap))
phecodemap$PhecodeLow2 = c(1:nrow(phecodemap))
phecodemap$PhecodeHigh2 = c(1:nrow(phecodemap))
phecodemap$PhecodeLow3 = c(1:nrow(phecodemap))
phecodemap$PhecodeHigh3 = c(1:nrow(phecodemap))

for(item in 1:length(broken)){
  phecodemap$PhecodeLow1[item] = broken[[item]][1] #high value of exclusion criteria
  phecodemap$PhecodeHigh1[item] = broken[[item]][2] #low value
  phecodemap$PhecodeLow2[item] = broken[[item]][3] #high value of exclusion criteria (if a list of exclusions exists)
  phecodemap$PhecodeHigh2[item] = broken[[item]][4] #low value
  phecodemap$PhecodeLow3[item] = broken[[item]][5] #high value of exclusion criteria (if a list of exclusions exists)
  phecodemap$PhecodeHigh3[item] = broken[[item]][6] #low value
}

#Remove rows for the glm() if an exclusion phecode is present (using Phecode Map 1.2 from https://phewascatalog.org/phecodes)
ExclusionCriterion = function(sleepdata, phecode, phemap) {
  
  PheMain = as.numeric(gsub('[A-Z]|[a-z]', '', phecode))
  PheExclLow1 = unique(phemap$PhecodeLow1[as.numeric(phemap$PheCode)==PheMain])
  PheExclHigh1 = unique(phemap$PhecodeHigh1[as.numeric(phemap$PheCode)==PheMain])
  PheExclLow2 = unique(phemap$PhecodeLow2[as.numeric(phemap$PheCode)==PheMain])
  PheExclHigh2 = unique(phemap$PhecodeHigh2[as.numeric(phemap$PheCode)==PheMain])
  PheExclLow3 = unique(phemap$PhecodeLow3[as.numeric(phemap$PheCode)==PheMain])
  PheExclHigh3 = unique(phemap$PhecodeHigh3[as.numeric(phemap$PheCode)==PheMain])
  
  #Maintain index for selection later
  StripIndex = c(1:ncol(sleepdata))
  MainIndex = StripIndex[colnames(sleepdata)==phecode]
  
  #Extract numeric phecodes
  StripCols = colnames(sleepdata)[grep('PheCode', colnames(sleepdata))]
  StripCols = gsub('PheCode', '', StripCols)
  #Trim indices
  StripIndex = StripIndex[grep('PheCode', colnames(sleepdata))]
  
  StripCols = as.numeric(StripCols[grep('[0-9]\\.[0-9]', StripCols)])
  if(is.finite(PheExclLow1) & is.finite(PheExclHigh1)){
    ExcludePositions = StripIndex[StripCols >= PheExclLow1 & StripCols <= PheExclHigh1] 
    ExcludePositions = c(ExcludePositions, StripIndex[StripCols >= PheExclLow2 & StripCols <= PheExclHigh2]) 
    ExcludePositions = c(ExcludePositions, StripIndex[StripCols >= PheExclLow3 & StripCols <= PheExclHigh3])
    ExcludePositions = ExcludePositions[is.finite(ExcludePositions)]
    ExcludePositions = ExcludePositions[c(ExcludePositions!=MainIndex)]
  } else {
    ExcludePositions = NULL
  }
  
  Modsleepdata = sleepdata
  #In rows where the main phecode is 0, but one of the other exclusion contains a 1, trim the row (i.e. they aren't a "control")
  if(length(ExcludePositions) > 0){
    for(exclude in ExcludePositions){
      Modsleepdata = Modsleepdata[-c(Modsleepdata[, MainIndex]==0 & Modsleepdata[, c(exclude)] == 1), ]
    }
  }
  return(Modsleepdata)
}

#Function to cycle through PheWAS for the sleep variables
PheWASFn = function(sleepdata, sleepvar, phecodes, phemap, group = '') {
  
  PValList = matrix(nrow = length(phecodes), ncol = 4)
  
  #For every phecode..
  for(i in 1 : length(phecodes)) {
    PheCode = phecodes[i]
    
    #Check an optional group call
    if (tolower(group) == 'males' | tolower(group) == 'females') {
      f = paste(PheCode, "~", sleepvar, "+ VisitAge + SD_RACE")
    } else if (tolower(group) == 'black' | tolower(group) == 'white') {
      f = paste(PheCode, "~", sleepvar, "+ VisitAge + GENDER_EPIC")
    } else {
      f = paste(PheCode, "~", sleepvar, "+ VisitAge + GENDER_EPIC + SD_RACE")
    }
    
    #Function to model a modified data frame, where exclusion phecodes are factored
    modsleepdata = ExclusionCriterion(sleepdata, PheCode, phemap)
    
    err = try(PheWASModel <- glm(f, data = modsleepdata, family = binomial(link = 'logit')), silent = TRUE)
    if(inherits(err, "try-error")) {
      PValList[i, ] = NaN
    } else {
      
      #Extract model info
      df = as.data.frame(summary(PheWASModel)$coefficients)
      CoefPr = df[grep('\\.', rownames(df)), ]
      colnames(PValList) = colnames(CoefPr)
      if(length(CoefPr) > 0) {
        PValList[i, 1] = CoefPr[1, 1]
        PValList[i, 2] = CoefPr[1, 2]
        PValList[i, 3] = CoefPr[1, 3]
        PValList[i, 4] = CoefPr[1, 4]
      }
      if(length(CoefPr) == 0) {
        PValList[i, ] = NaN
      }
    }
  }
  
  #Processing/export
  SnpTrimExport = cbind(1 : nrow(PValList), PValList)
  SnpTrimExport = as.data.frame(SnpTrimExport)
  SnpTrimExport = cbind(phecodes, SnpTrimExport)
  SnpTrimExport = SnpTrimExport[ , c(1, 3, 4, 5, 6)]
  SnpTrimExport = SnpTrimExport[order(SnpTrimExport$`Pr(>|z|)`), ]
  
  #Multiple test adjust
  SnpTrimExport$FDR = p.adjust(SnpTrimExport$`Pr(>|z|)`, method = 'fdr')
  SnpTrimExport$Bonf = p.adjust(SnpTrimExport$`Pr(>|z|)`, method = 'bonf')
  SnpTrimExport$phecodes = gsub("PheCode", "", SnpTrimExport$phecodes)
  
  #Add OR and 95% CI
  SnpTrimExport$OR = exp(SnpTrimExport$Estimate)
  SnpTrimExport$LowCI = exp(SnpTrimExport$Estimate + qnorm(c(0.025)) * SnpTrimExport$'Std. Error')
  SnpTrimExport$HighCI = exp(SnpTrimExport$Estimate + qnorm(c(0.975)) * SnpTrimExport$'Std. Error')
  
  SnpTrimExport = merge(SnpTrimExport, pheinfo,by.x = "phecodes", by.y = "phecode")
  SnpTrimExport = SnpTrimExport[order(SnpTrimExport$`Pr(>|z|)`), ]
  
  #PheWAS plot
  PheWASPlot = QQPlot(SnpTrimExport, pheinfo, sleepvar, group)
  return(PheWASPlot)
}

WeekendPlot = PheWASFn(ComboDF, 'Weekend.Sleep.Duration', PheList, phecodemap)
WeekendPlot2 = WeekendPlot + theme(plot.margin = margin(t = 0.1, r = 0.5, b = 0, l = 0),
                                   axis.title.y = element_text(margin = margin(t = 0, r = 0, b = 0, l = -0.1)))

WeekdayPlot = PheWASFn(ComboDF, 'Weekday.Sleep.Duration', PheList, phecodemap)
WeekdayPlot2 = WeekdayPlot + ylab('')  + theme(plot.margin = margin(t = 0.1, r = 0.5, b = 0, l = 0),
                                   axis.title.y = element_text(margin = margin(t = 0, r = 0, b = 0, l = -0.1)))

SocialPlot = PheWASFn(ComboDF, 'Social.Sleep.Lag', PheList, phecodemap)
SocialPlot2 = SocialPlot + ylab('') + theme(plot.margin = margin(t = 0.1, r = 0.5, b = 0, l = 0),
                                 axis.title.y = element_text(margin = margin(t = 0, r = 0, b = 0, l = -0.1)))

CowPlot = plot_grid(WeekendPlot2, WeekdayPlot2, SocialPlot2, ncol = 3, scale = 0.95, align = 'vh', labels = "AUTO", hjust = c(-0.5, -0.5, -0.5), vjust = c(1.25, 1.25, 1.25), label_size = 14)

ggsave(filename = "FigS4.pdf", plot = CowPlot, path = file.path(filepath, 'PheWAS'), width = 8, height = 3, units = c("in"))
